@@ -141,7 +141,8 @@ class Framework:
                 "keyPrefix": {
                     "eval": "#",
                     "exec": "%",
-                    "loop": "!"
+                    "loop": "!",
+                    "shell": "$",
                 },
                 "loadingFiles": {
                     "ctx": "ctx.yaml",
@@ -155,6 +156,12 @@ class Framework:
                 }
             }
         })
+        cfg = render(
+            cfg,
+            peval=cfg["framework"]["keyPrefix"]["eval"],
+            pexec=cfg["framework"]["keyPrefix"]["exec"],
+            pshell=cfg["framework"]["keyPrefix"]["shell"],
+        )
 
         if "i18n" in cfg:
             cfg["reporter"][reporter] = merge(cfg["reporter"][reporter], cfg["i18n"])
@@ -236,6 +243,7 @@ class Framework:
         info = Framework.load_ctx(os.path.basename(directory), "{}/{}".format(directory, customize.loadingFiles.ctx))
         description = info["description"] + Framework.load_description("{}/{}".format(directory, customize.loadingFiles.description))
         var_info = copy.deepcopy(parent_rctx.var_info) | info["var"] | Framework.load_var("{}/{}".format(directory, customize.loadingFiles.var))
+        var_info = render(var_info, peval=customize.keyPrefix.eval, pexec=customize.keyPrefix.exec, pshell=customize.keyPrefix.shell)
         var = json.loads(json.dumps(var_info), object_hook=lambda x: SimpleNamespace(**x))
         common_step_info = copy.deepcopy(parent_rctx.common_step_info) | info["commonStep"] | Framework.load_common_step("{}/{}".format(directory, customize.loadingFiles.commonStep))
         before_case_info = copy.deepcopy(parent_rctx.before_case_info) + info["beforeCase"] + list(Framework.load_step("{}/{}".format(directory, customize.loadingFiles.beforeCase)))
@@ -259,7 +267,7 @@ class Framework:
                     },
                 },
             })
-            val = render(val, var=var, x=parent_rctx.x, peval=customize.keyPrefix.eval, pexec=customize.keyPrefix.exec)
+            val = render(val, var=var, x=parent_rctx.x, peval=customize.keyPrefix.eval, pexec=customize.keyPrefix.exec, pshell=customize.keyPrefix.shell)
             ctx[key] = parent_rctx.driver_map[val["type"]](val["args"])
             dft[key] = val["dft"]
 
@@ -620,6 +628,9 @@ class Framework:
         step = StepResult(step_info["name"], step_info["ctx"], step_info["description"])
         now = datetime.now()
 
+        # use json transform tuple to list
+        step_info["req"] = render(json.loads(json.dumps(step_info["req"])), case=case, var=rctx.var, x=rctx.x, peval=customize.keyPrefix.eval, pexec=customize.keyPrefix.exec, pshell=customize.keyPrefix.shell)
+
         mode = ""
         if customize.keyPrefix.eval + "res" in step_info:
             step_info["res"] = step_info[customize.keyPrefix.eval + "res"]
@@ -648,10 +659,10 @@ class Framework:
                     step.add_sub_step_result(result)
 
         # auto name step
-        if not step.name:
+        if not step.name and step.req:
             step.name = rctx.ctx[step_info["ctx"]].name(step.req)
-            if not step.name:
-                step.name = "anonymous-step"
+        if not step.name:
+            step.name = "anonymous-step"
         step.elapse = datetime.now() - now
         return step
 
@@ -661,8 +672,6 @@ class Framework:
         sub_step_start = datetime.now()
         try:
             req = merge(req, rctx.dft[step_info["ctx"]]["req"])
-            # use json transform tuple to list
-            req = render(json.loads(json.dumps(req)), case=case, var=rctx.var, x=rctx.x, peval=customize.keyPrefix.eval, pexec=customize.keyPrefix.exec)
             sub_step_result.req = req
 
             retry = Retry(merge(step_info["retry"], rctx.dft[step_info["ctx"]]["retry"]))
